-- CityAdoptReligionEvent
-- Author: Machiavelli
-- DateCreated: 8/20/2012 10:02:01 AM
--------------------------------------------------------------
-- Purpose:
-- LuaEvents.CityAdoptsReligionEvent(iOwner, iX, iY, eOldReligion, eNewReligion, bFirstConversion)
-- Will be called whenever any city (human or ai) adopts or changes to a religion.  It is not called
-- when a Pantheon is adopted.  Nor is it called when an Inquisitor cleans a city.
--
-- Known limitations:
-- When an Inquisitor cleans a city the religion building marking that city remains even though
-- the city is no longer that religion.  The religion building won't be removed until this
-- function is called again, and if the city has at least 1 follower of the old religion when
-- that happens (due to natural pressure or a missionary) the CityAdoptsReligionEvent won't fire.
--
-- iOwner: The ID of the player who owns the city
-- iX, iY: The location of the city, use Map.GetPlot(iX, iY) to get the plot object
-- eOldReligion: The ID of the religion the city use to be.  If the city was not previously a
--       founded religion (due to the city having a Pantheon or no Pantheon) eOldReligion will
--       be 0.
-- eNewReligion: The ID of the religion the city is now.  Will always be 1 or greater because
--       the 0 ID is for "Pantheon religions".
-- bFirstConversion: A boolean that will be true if this is the first time the city has become
--       eNewReligion, false otherwise.
--------------------------------------------------------------
function CallCityAdoptsReligionEvent(iOwner, eReligion, iX, iY)

	local plot = Map.GetPlot(iX, iY);
	-- Error check, if this isn't a city don't continue
	if(not plot:IsCity()) then
		return;
	end

	local city = plot:GetPlotCity();
	local eNewReligion = city:GetReligiousMajority();
	local newReligionBuilding = nil;

	-- If a city has no religious majority, eNewReligion will be -1
	if(eNewReligion < 1) then
		-- Remove any buildings marking what the old religion the city is (TODO: this check should be happening every turn to avoid having incorrect state for a few turns)
		for row in GameInfo.Religions() do
			local temp = "BUILDING_" .. row.Type;

			if (city:IsHasBuilding(GameInfoTypes[temp])) then
				city:SetNumRealBuilding(GameInfoTypes[temp], 0); -- Remove the building
			end
		end
		-- Do not continue because the city has no majority religion
		return;
	else
		newReligionBuilding = "BUILDING_" .. GameInfo.Religions[eNewReligion].Type;
		if(city:IsHasBuilding(GameInfoTypes[newReligionBuilding])) then
			-- The city has not changed religion, don't continue
			return;
		end
	end

	-- Only way to get here is if: eNewReligion >= 1 and city does not have newReligionBuilding
	local eOldReligion = 0;
	-- Find any buildings marking what the old religion the city is
	for row in GameInfo.Religions() do
		local temp = "BUILDING_" .. row.Type;
		-- Skip the current local majority religion (eNewReligion) but remove all other religion marking buildings
		if (row.ID ~= eNewReligion and city:IsHasBuilding(GameInfoTypes[temp])) then
			city:SetNumRealBuilding(GameInfoTypes[temp], 0);

			if(city:GetNumFollowers(row.ID) > 0 or city:GetPopulation() == 1) then
				-- Possible error check, if this is the second religion marking building found, could cause issues.
				-- Found the old religion of the city
				eOldReligion = row.ID;
			end
		end
	end

	-- Is this the first time the city has adopted the majorityReligion
	local hasBeenReligionBuilding = "BUILDING_HAS_BEEN_" .. GameInfo.Religions[eNewReligion].Type;
	local bFirstConversion = false;

	if(not city:IsHasBuilding(GameInfoTypes[hasBeenReligionBuilding])) then
		bFirstConversion = true;
	end

	-- Update religion building state
	city:SetNumRealBuilding(GameInfoTypes[newReligionBuilding], 1);
	city:SetNumRealBuilding(GameInfoTypes[hasBeenReligionBuilding], 1);

	-- Call the lua event
	print("Calling CityAdoptsReligionEvent(iOwner: " .. tostring(iOwner) .. " iX: " .. tostring(iX) .. " iY: " .. tostring(iY) .. " eOldReligion: " .. tostring(eOldReligion) .. " eNewReligion: " .. tostring(eNewReligion) .. " bFirstConversion: " .. tostring(bFirstConversion) .. ")");
	LuaEvents.CityAdoptsReligionEvent(iOwner, iX, iY, eOldReligion, eNewReligion, bFirstConversion);
end
GameEvents.CityConvertsReligion.Add(CallCityAdoptsReligionEvent);